import os 

import FreeCAD as App
from . import settings
import Part

from kipy.board_types import Footprint3DModel, BoardPolygon, BoardSegment, PadStackShape
from kipy.geometry import PolygonWithHoles, PolyLine, PolyLineNode, Vector2
from kipy.proto.common.types import KiCadObjectType
from kipy.util.board_layer import BoardLayer

from .bases import BaseObject, BaseViewProvider

class BoardSketchObject(BaseObject):
    TYPE = 'KiConnect::BoardSketch'

    def __init__(self, feature, kicad_board, board_polygon):
        super(BoardSketchObject, self).__init__(feature)

        #feature.Visibility = False

        vectors = []

        for node in board_polygon.polygons[0].outline:
            vectors.append(self.point_to_vector(node.point))

        self.feature.Vectors = vectors

    def execute(self, feature):
        begin = None
        start = None

        # this probably needs a bit more control..
        feature.Constraints = []
        feature.Geometry = []

        for vector in self.feature.Vectors:
            if not start:
                start = vector
                begin = vector

                continue

            feature.addGeometry(Part.LineSegment(start, vector))
            start = vector

        feature.addGeometry(Part.LineSegment(start, begin))

        feature.recompute()

    def point_to_vector(self, point, offset=True):
        return (
            App.Vector(point.x,
                       -point.y
            )) / 1000000.0 - (self.feature.BoardOffset.Base if offset else 0)

    def setup_properties(self, feature):
        super(BoardSketchObject, self).setup_properties(feature)

        feature.addProperty('App::PropertyPlacement', 'BoardOffset', 'KiConnect', 'Internal offset for zeroing out Footprint offset', hidden=True, read_only=True)
        feature.addProperty('App::PropertyVectorList', 'Vectors', 'KiConnect', 'Internal offset for zeroing out Footprint offset', hidden=True)

    def __getstate__(self):
        return None

class BoardSketchViewProvider(BaseViewProvider):
    TYPE = 'KiConnect::BoardSketch'
    #ICON = 'board.svg'

def makeBoardSketch(parent, kicad_board, polygon):
    feature = App.ActiveDocument.addObject('Sketcher::SketchObjectPython', 'BoardSketch')
    parent.addObject(feature)

    BoardSketchObject(feature, kicad_board, polygon)
    BoardSketchViewProvider(feature.ViewObject)

    return feature
